<?php

namespace Avos\Database;

$dbh = mysqli_connect(DB_HOST, DB_USER, DB_PASS, DB_NAME);

if ($dbh === false) {
	echo "unable to connect\n";
	exit;
}

$found_rows = 0;
$caching = true;
$cacheTime = 0;

function dbh()
{
	global $dbh;
	return $dbh;
}

function disableCache()
{
	global $caching;
	$caching = false;
}

function enableCache()
{
	global $caching;
	$caching = true;
}

function setCacheTime($t)
{
	global $cacheTime;
	$cacheTime = $t;
}

function query($sql, $args=array())
{
	global $dbh, $_AVOS;

	//
	// choose which database to use
	//

	//
	// expand the args
	//

    $osql = $sql;

	if (count($args) > 0)
	{
		$regex = array();
		$replace = array();
		#echo "BEFORE: $sql\n";
		#echo "ARGS  : "; print_r($args);

		foreach ($args as $k => $v)
		{ 
			if (!is_array($v))
			{
				$regex[] = '!:' . $k . '\b!';
				$replace[] = "'" . mysqli_real_escape_string($dbh, $v) . "'";
			}
		}

		$sql = preg_replace($regex, $replace, $sql);
		#echo "AFTER : $sql\n";
	}

	//
	// prefix webcams and webcamsArchive with the database name (could be the default or a shared one)
	//

	$sql = preg_replace('!(FROM\s+|JOIN\s+)(webcams[\b\n\r\s\)]+)!i', '$1' . DB_CAMS . '.$2', $sql);
	$sql = preg_replace('!(FROM\s+)(webcams)$!i', '$1' . DB_CAMS . '.$2', $sql);
    $sql = preg_replace('!(.*,\s*)(webcams w.*)!', '$1' . DB_CAMS . '.$2', $sql);
    $sql = str_replace('webcamsArchive', DB_CAMS . '.webcamsArchive', $sql);
    $sql = str_replace('FROM history', 'FROM ' . DB_CAMS . '.history', $sql);
	#echo "<!--\n$sql\n-->\n";
	#echo "<h1>$sql</h1>";
	#echo "<p>$sql</p>";

	//
	// issue msyql query
	//

	#echo "$sql    <br>\n";
	#exit;
	#if (DEBUG) echo "$sql\n";
	#echo "$sql\n";
	#echo "<!-- $sql -->\n";

	\Avos\Utils\profFlag("$sql");
	$db = mysqli_query($dbh, $sql);
	$error =  mysqli_error($dbh);

	if (!empty($error)) {
		echo "<!--\nERROR: $error\n\n$sql\n\n$osql\n\n"; print_r($args); echo "\n-->\n";
		$retries = 5;

		if (strpos($error, "Deadlock") !== false) {
			while (strpos($error, "Deadlock") !== false && $retries-- > 0) {
				$db = mysqli_query($dbh, $sql);
				$error =  mysqli_error($dbh);
			}
		}

		if (empty($error)) {
			echo "<!--\nSUCCESS on retry " . (5-$retries) . "\n-->\n";
		}
	}

	//
	// return results or insert id
	//

	if (preg_match('/^\s*(REPLACE|INSERT)/i', $sql))
	{
		$id = mysqli_insert_id($dbh);
		#$id = $dbh->insert_id;
		return $id;
	}
	else {
		return $db;
	}
}

function queryRow($sql, $args=array())
{
	$db = query($sql, $args);
	if ($db === FALSE) return array();

	$ret = mysqli_fetch_row($db);

	return $ret;
}

function queryList($sql, $args=array())
{
	$db = query($sql, $args);
	if ($db === FALSE) return array();

	$ret = array();
	while ($tmp = mysqli_fetch_array($db, MYSQLI_NUM)) $ret = array_merge($ret, $tmp);

	return $ret;
}

function queryListQuoted($sql, $args=array())
{
	$ret = queryList($sql, $args);

	for ($i=0; $i<count($ret); $i++)
	{   
		$ret[$i] = quote($ret[$i]);
	}

	return $ret;
}

function queryAssoc($sql, $args=array())
{
	global $found_rows, $_AVOS, $caching, $cacheTime;
	#$_AVOS['redis']->delete($sql);

	//
	// see if we have it cached in redis
	//

	if (isset($_AVOS['config']['redis']['prefix'])) {
		$key = 'sql/' . $_AVOS['config']['redis']['prefix'] . '/' . md5($sql);
	}
	else {
		$key = 'sql/' . md5($sql);
	}

	if ($caching == true && preg_match('/^\s*SELECT/i', $sql)) {
		if (isset($_AVOS['redis'])) {
			$ret = $_AVOS['redis']->get($key);
			if ($ret !== FALSE) {
				$found_rows = $_AVOS['redis']->get($key . '/rows');
				return $ret;
			}
		}
	}

	$db = query($sql, $args);
	if ($db === FALSE) return array();

	$ret = array();
	while ($tmp = mysqli_fetch_assoc($db)) array_push($ret, $tmp);

	//
	// cache in redis
	//

	if ($caching == true && preg_match('/^\s*SELECT/i', $sql)) {
		if (isset($_AVOS['redis'])) {
			$_AVOS['redis']->set($key, $ret);

			if ($cacheTime > 0) {
				$_AVOS['redis']->expire($key, $cacheTime);
			}
			else {
				$_AVOS['redis']->expire($key, $_AVOS['config']['redis']['expiry']);
			}

			if (strpos($sql, 'SQL_CALC_FOUND_ROWS') !== false) {
				list($found_rows) = queryList('SELECT FOUND_ROWS()');
				$_AVOS['redis']->set($key . '/rows', $found_rows);
				$_AVOS['redis']->expire($key . '/rows', $_AVOS['config']['redis']['expiry']);
			}
		}
	}

	return $ret;
}

function foundRows()
{
	global $found_rows;
	return $found_rows;
}

function quote($str)
{
	global $dbh;
	return "'" . mysqli_real_escape_string($dbh, $str) . "'";
}

function quoteLike($str)
{
	global $dbh;
	return "'%" . mysqli_real_escape_string($dbh, $str) . "%'";
}
