<?

//
// general purpose template functions
//

function getConfig($block=null, $value=null, $defaultValue=true)
{
	global $_AVOS;

	if (!empty($block)) {
        if (isset($value)) {
			if (isset($_AVOS['config'][$block][$value])) {
				return $_AVOS['config'][$block][$value];
			}
            else {
				return $defaultValue;
			}
		}
		else {
			return $_AVOS['config'][$block];
		}
	}
	else {
		return $_AVOS['config'];
	}
}

function redirect($url)
{
	if (strpos($url, '?') == false && count($_GET) > 0) {
		$url .= '?' . http_build_query($_GET);
	}

	header("Cache-Control: no-cache, no-store, must-revalidate"); // HTTP 1.1.
	header("Pragma: no-cache"); // HTTP 1.0.
	header("Expires: 0"); // Proxies.
	header("Location: $url", true, 301);
	exit;
}

function pre($e, $exit=true)
{
    echo "<pre>";
    print_r($e);
    if ($exit) exit;
}

function parseTemplate($name, $args=array())
{
	return \Avos\Scaffold\parsePhpTemplate($name, $args);
}

function setTemplateValue($k, $v)
{
	global $_AVOS;
	#echo "setTemplateValue($k, $v)<br>";
	$_AVOS['templateVars'][$k] = $v;
}

function getTemplateValue($k)
{
	global $_AVOS;

	if (isset($_AVOS['templateVars'][$k])) {
		return $_AVOS['templateVars'][$k];
	}
	else {
		return false;
	}
}

function getTemplateValues()
{
    global $_AVOS;

    if (isset($_AVOS['templateVars'])) {
        return $_AVOS['templateVars'];
    }
    else {
        return [];
    }
}

function getUser()
{
    return \Avos\Users\getUser();
}

function isLoggedIn()
{
    return \Avos\Users\isLoggedIn();
}

function isModel()
{
    return \Avos\Users\isModel();
}

function getUrl($name = '')
{
    global $_AVOS;

    if (!empty($name)) {

		//
		// use the matching [urls] rule
		//

        if (empty($_AVOS['config']['urls'][$name])) {
            return '';
        }

		$url = $_AVOS['config']['urls'][$name];
		$url = explode(',', $url);
        $url = trim($url[1], '[] ');

        return $url;
    }

	return getUrlMatched();
}

function getUrlMatched()
{
	global $_AVOS;
	return $_AVOS['page'];
}

function setUrl($url)
{
	setUrlMatched($url);
}

function setUrlMatched($url)
{
	global $_AVOS;
	return $_AVOS['page'] = $url;
}

function getCanonicalUrl()
{
    global $_AVOS;

    $url = rtrim(getConfig('site', 'url'), '/') . encodeUrl($_AVOS['page']['name'], $_AVOS['page']['args']);
    $qs = $_SERVER['QUERY_STRING'];

    if (!empty($qs)) {
        $url .= '?' . $qs;
    }

    return $url;
}

function getSiteShortName($site)
{
	return \Avos\Scaffold\shortSiteName($site);
}

function getSiteLongtName($site)
{
	return \Avos\Scaffold\longSiteName($site);
}

function getLongGender($g)
{
	return \Avos\Scaffold\expandGender($g);
}

function encodeUrl($name, $replace=array(), $tolower=null)
{
	global $_AVOS;

	if (!isset($replace['page'])) {
		$replace['page'] = '';
    }

	// delete any non-scalars
	foreach ($replace as $k => $v) {
		if (is_array($v)) {
			$replace[$k] = '';
		}
	}

	if ($name == null) {
		//
		// use the current route
		//

		$url = $_AVOS['page']['route'];
	}
	elseif (isset($_AVOS['config']['urls'][$name])) {
		//
		// use the matching [urls] rule
		//

		$url = $_AVOS['config']['urls'][$name];
		$url = explode(',', $url);
		$url = trim($url[1], '[] ');
	}
	else {
		//
		// use $name as the url itself
		//

		$url = $name;
	}

	$search = array();
	foreach ($replace as $k => $v) {
		$search[] = ':' . $k;
	}

    $url = str_replace($search, $replace, $url);

    if (($tolower === null && getConfig('site', 'lowerCaseUrls')) || $tolower) {
        $url = strtolower($url);
    }

	return $url;
}

function getRoute()
{
    global $_AVOS;

    $route =  $_AVOS['page']['route'];

    if (!empty($_SERVER['QUERY_STRING'])) {
        $route .= '?' . $_SERVER['QUERY_STRING'];
    }

    if (!strpos($route, ':page') && getConfig('templates', 'homePagination', null) !== null) {
        $route = str_replace(':page', $page, getConfig('templates', 'homePagination', '/'));
    }

    return $route;
}

//
// webcams template functions
//

function getWebcamsCounts()
{
	global $_AVOS;

	$key = \Avos\Redis\getPrefix('webcams/counts');
	return $_AVOS['redis']->get($key);
}

function getCounts()
{
    return getWebcamsCounts();
}

function getSites()
{
    $sites = getWebcamsCounts()['sites'];
    $filter = getConfig('webcams', 'sitesFilter', false);

    if (!$filter) {
        return $sites;
    }

    $haveSite = [];
    foreach (explode(',', $filter) as $s) {
        $haveSite[$s] = 1;
    }

    $ret = [];

    foreach ($sites as $s) {
        if (isset($haveSite[ getSiteShortName($s['site']) ])) {
            $ret[] = $s;
        }
    }

    return $ret;
}

function getCatTag($type, $sort='')
{
    $t = getWebcamsCounts()[$type];

    if (empty($sort)) {
        return $t;
    }

    $c = array_column($t, $sort);
    $o = SORT_DESC;

    if ($sort == 'category' || $sort == 'tag') {
        $o = SORT_ASC;
    }

    array_multisort($c, $o, $t);

    return $t;
}

function getCategories($sort='')
{
    return getCatTag('categories', $sort == 'name' ? 'category' : $sort);
}

function getTags($sort='')
{
    return getCatTag('tags', $sort == 'name' ? 'tag' : $sort);
}

function encodePagination($page)
{
	global $_AVOS;

    $route = $_AVOS['page']['route'];

    if (strpos($route, ':page')) {
        $route = str_replace(':page', $page, $route);
    }
    elseif (getConfig('templates', 'homePagination', null) !== null) {
        $route = str_replace(':page', $page, getConfig('templates', 'homePagination', '/'));
    }
    else {
        $route = $route . "?page=$page";
    }

    $qs = \Avos\Utils\url2array($_SERVER['QUERY_STRING']);

	if (preg_match_all('!:(\w+)!', $route, $c)) {
		foreach ($c[1] as $p) {
			if (isset($_AVOS['page'][$p])) {
                $route = str_replace(':' . $p, $_AVOS['page'][$p], $route);
                unset($qs[$p]);
			}
		}

	}

	$route = strtolower($route);

    /*
    foreach (['search', 'kw', 'q'] as $kw) {
        if (isset($_GET[$kw])) {
            $route .= "?$kw=" . $_GET[$kw];
        }
    }
     */

    if (isset($qs['layout'])) {
        unset($qs['layout']);
    }

    if (count($qs) > 0) {
        if (strpos($route, '?') !== false) {
            $route .= '?';
        }
        $route .= \Avos\Utils\array2url($qs);
    }

	return $route;
}

function encodeRoomLink($site, $nickname='')
{
	global $_AVOS;

	if (is_array($site)) {
		$nickname = $site['nickname'];
		$site = $site['site'];
	}

    $database = getConfig('mysql', 'database', '');
    $tracking = getConfig('site', 'tracking', $database);

	$site = \Avos\Scaffold\longSiteName($site);
	$url = $_AVOS['config']['webcams'][$site]['roomLink'];
	$url = str_replace(':nickname', $nickname, $url);
	$url = str_replace(':affiliate', $_AVOS['config']['webcams'][$site]['affiliate'], $url);
    $url = str_replace(':tracking', $tracking, $url);

	return $url;
}


function encodeChatEmbed($model, $srcOnly = 0)
{
    global $_AVOS;

    if ($srcOnly) {
        $elt = 'chatEmbedLink';
    }
    else {
        $elt = 'chatEmbed';
    }

    // in case we need streamUrl

    $m = getWebcams(['site' => $model['site'], 'nickname' => $model['nickname']]);
    if (isset($m[0]['streamUrl'])) {
        $model['streamUrl'] = $m[0]['streamUrl'];
    }

    $database = getConfig('mysql', 'database', '');
    $tracking = getConfig('site', 'tracking', $database);

    $site = \Avos\Scaffold\longSiteName($model['site']);
    $html = $_AVOS['config']['webcams'][$site][$elt];

	$html = str_replace(':nickname', $model['nickname'], $html);
	$html = str_replace(':affiliate', $_AVOS['config']['webcams'][$site]['affiliate'], $html);
	$html = str_replace(':tracking', $tracking, $html);

	if (!empty($model['externalId'])) {
		$html = str_replace(':externalId', $model['externalId'], $html);
	}



	if (isset($model['streamUrl'])) {
		$html = str_replace(':streamUrl', $model['streamUrl'], $html);
	}

	// might also need :roomLink
	if (strpos($html, ':roomLink') !== false) {
		$url = $_AVOS['config']['webcams'][$site]['roomLink'];
		$url = str_replace(':nickname', $model['nickname'], $url);
		$html = str_replace(':roomLink', $url, $html);
	}

	return $html;
}

function getWebcamsCategory($category, $type='categories')
{
	$cats = listWebcamsCategories($type);
	$single = $type == 'categories' ? 'category' : 'tag';

	foreach ($cats as $c) {
		if ($c[$single] == $category) {
			return $c;
		}
	}

	return FALSE;
}

function listWebcamsCategories($type='categories')
{
	global $_AVOS;

	$single = $type == 'categories' ? 'category' : 'tag';

	if (isset($_AVOS['config']['webcams'][$type])) {

		$ret = array();

		//
		// all we need is the name and urlname
		//

        foreach ($_AVOS['config']['webcams'][$type] as $n => $c) {

            $c = explode(',', trim($c, '[] '));
            $urlname = $c[0];
            $filter = $c[1];

            $ret[] = [
                'name' => $n,
                'fullname' => $n,
                $single => trim($c[0], ' '),
                'filter' => trim($c[1], ' '),
                'gender' => isset($c[2]) ? trim($c[2], ' ') : ''
            ];
        }

		return $ret;

	}
	else {
		return array();
	}
}

function getActivityFeed($m, $months=2)
{
	return \Avos\Webcams\getActivityFeed($m, $months);
}

function getHeatMap($m)
{
	return \Avos\Webcams\getHeatMap($m);
}

function getDailyHours($m, $months=0)
{
	return \Avos\Webcams\getDailyHours($m, $months);
}

function getRanks($m, $n=0, $t=0)
{
	return \Avos\Webcams\getRanks($m, $n, $t);
}

function expandUrlMeta($data)
{
	\Avos\Scaffold\expandUrlMeta($data);
}

function getWebcams($q='', $a=array(), $csql='', $archive='')
{
    if (!is_array($q)) {
        parse_str($q, $arr);
    }
    else {
        $arr = $q;
    }

	if ($a == null) {
		$a = array();
	}

	foreach (array('count', 'page', 'sort') as $k) {
		if (isset($arr[$k])) {
			$a[$k] = $arr[$k];
			unset($arr[$k]);
		}
	}

    $sql = \Avos\Utils\query2sql($arr);

	// append any custom sql if provided
	if ($csql != null && $csql != '') {
		if ($sql != '') {
			$sql .= ' AND ';
		}
		$sql .= $csql;
    }

    $webcams = \Avos\Routes\webcams($a, $sql, $archive);

	if (isset($webcams['args']) && isset($webcams['args']['models'])) {
		return $webcams['args']['models'];
	}
	else {
		return array();
	}
}

function getWebcam($q='', $a=array(), $csql='', $archive='')
{
    global $_AVOS;

    // save page matches/pages for main webcams search
    $page = $_AVOS['page'];

    $q['count'] = 1;
    $models = getWebcams($q, $a, $csql, $archive);

    // restore page matches/pages for main webcams search
    $_AVOS['page'] = $page;

    return $models;
}

function getWebcamsLite($q='', $a=array(), $csql='', $archive='')
{
    $models = getWebcams($q, $a, $csql, $archive);
    $ret = [];

    foreach ($models as $m) {
        $ret[] = [
            'id' => $m['id'],
            'nickname' => $m['nickname'],
            'subject' => $m['subject'],
            'viewers' => $m['viewers'],
            'followers' => $m['followers'],
            'tags' => $m['tags'],
            'site' => $m['site'],
            'status' => $m['status'],
            'age' => $m['age'],
            'gender' => $m['gender'],
            'imageUrl' => $m['imageUrl'],
            'imagePath' => $m['imagePath'],
            'numImages' => $m['numImages'],  
            'siteShortNameCaps' => $m['siteShortNameCaps']
        ];
    }

    return $ret;
}


function getVideos($q='', $a=array())
{
	global $_AVOS;

    if (!is_array($q)) {
        parse_str($q, $arr);
    }
    else {
        $arr = $q;
    }

	// remove attribs that arent part of the sql filtering

	foreach (array('count', 'sort') as $f) {
		if (isset($arr[$f])) {
			$a[$f] = $arr[$f];
			unset($arr[$f]);
		}
	}

	// corner case where url type=webcams and the userfn calls getVideos()

	$type = $_AVOS['page']['type'];
	$_AVOS['page']['type'] = 'videos';

	$sql = \Avos\Utils\query2sql($arr);
	$videos = \Avos\Routes\videos($a, $sql);

	$_AVOS['page']['type'] = $type;

	if (isset($videos['args']) && isset($videos['args']['videos'])) {
		return $videos['args']['videos'];
	}
	else {
		return array();
	}
}









function getRelatedWebcams($cats, $count=8, $type='categories', $catstags=false, $recursed=false)
{
    global $_AVOS;

	\Avos\Database\disableCache();
    $model = getTemplateValue('model');
    $counts = getTemplateValue('counts');
    $cats = str_replace(' ', '', $cats);
    $ocats = $cats;
	$cats = explode(',', $cats);
    $redis = $_AVOS['redis'];
    $otype = ($type == 'categories' ? 'tags' : 'categories');

    if (empty($counts)) {
        $counts = getWebcamsCounts();
    }

    // model is offline
    if ($model['status'] == 'Offline') {
        foreach ($cats as $c) {
            if (isset($counts[$type][$c]) && is_array($counts[$type][$c]['models'])) {
                $models = array_splice($counts[$type][$c]['models'], 0, $count);
				return $models;
			}
        }
        return [];
    }

    // if count is a query string extract the count and remaining filter
    $site = '';
    if (!is_int($count)) {
        parse_str($count, $filter);
        $count = 8;
        if (!empty($filter['count'])) {
            $count = $filter['count'];
        }
        if (!empty($filter['site'])) {
            $site = \Avos\Scaffold\longSiteName($filter['site']);
        }
    }

	// fetch one more than requested in case the current model is there
	$count++;

	// grab the webcams counts
	$categories = $counts[$type];
    $table = 'webcams' . ucfirst($type);

	$tmp = \Avos\Database\queryAssoc("SELECT name FROM $table WHERE webcamsId=:id", $model);

	// return the final category/tag if the model isnt a member of any

    if (count($tmp) == 0) {

        if ($recursed == false) {
            return getRelatedWebcams($ocats, $count-1, $otype, false, true);
        }

        // if recursed then switch back to the original cat/tag type
        if ($recursed == true) {
            $categories = $counts[$otype];
        }

		$cats = array_reverse($cats);
		foreach ($cats as $c) {
            if (isset($categories[$c]) && is_array($categories[$c]['models'])) {
                #$models = array_splice($categories[$c]['models'], 0, $count-1);
                // randomize the models list
                $models = stripGenders($model['gender'], $categories[$c]['models']);
                shuffle($models);
                $models = array_splice($models, 0, $count-1);
				return $models;
			}
        }

        return array();
	}

    // create map of all cats/tags the model is in
	$incat = array();
	foreach ($tmp as $c) {
		$incat[$c['name']] = 1;
    }

    // if $catstags=true and type=tags then fetch the 1st category the model is in
    $catstagscat = '';
    if ($catstags && $type == 'tags') {
        $tmp = \Avos\Database\queryList("SELECT name FROM webcamsCategories WHERE webcamsId=:id LIMIT 1", $model);
        if (count($tmp) == 1) {
            $catstagscat = $tmp[0];
        }
    }

	// check every category for a match
    foreach ($cats as $c) {

		if (!isset($incat[$c])) {
			continue;
        }

		if (isset($categories[$c])) {

            // either fetch the list of models in the tag or category 
            // or catstags

            if (!empty($catstagscat)) {
                $tmp = $counts['catstags'][$catstagscat][$c]['models'];
            }
            else {
                $tmp = $categories[$c]['models'];
            }

            $tmp = stripGenders($model['gender'], $tmp);

            if (count($tmp) > 9) {

                // randomize the models list
                shuffle($tmp);

                // need to ensure the current model isnt in the related list
                $ret = array();
                $i = 0;

                while (count($tmp) > 0 && count($ret) < $count-1) {
                    $m = array_pop($tmp);
                    if ($m['id'] != $model['id']) {
                        if (empty($site) || $site == $m['site']) {
                            $ret[] = $m;
                        }
                    }
                }

                return $ret;
            }
		}
    }

    if ($recursed == false) {
        return getRelatedWebcams($ocats, $count-1, $otype, $catstags, true);
    }
    else {
	    // if here then the model is in cats/tags but not in the list provided
        // if recursed then switch back to the original cat/tag type

        if ($recursed == true) {
            $categories = $counts[$otype];
        }

		$cats = array_reverse($cats);
		foreach ($cats as $c) {
            if (isset($categories[$c]) && is_array($categories[$c]['models'])) {
                #$models = array_splice($categories[$c]['models'], 0, $count-1);
                // randomize the models list
                $models = $categories[$c]['models'];
                shuffle($models);
                $models = array_splice($models, 0, $count-1);
				return $models;
			}
        }

        return array();
    }
}

function stripGenders($gender, $models)
{
    $ret = [];

    foreach ($models as $m) {
        if ($m['gender'] == $gender) {
            $ret[] = $m;
        }
    }

    return $ret;
}

function getVideosTags()
{
	return \Avos\Database\queryAssoc("SELECT name AS tag, COUNT(*) AS count FROM videosTags GROUP BY name");
}

function getBlogs()
{
	return \Avos\Database\queryAssoc("SELECT * FROM ArticleBlogPosts ");
}

function getPromote()
{
	return \Avos\Database\queryAssoc("SELECT * FROM promote ");
}


// promote my models |site |events |related 


    








function getFavoriteBlogs($ids) {
    $ids = implode(',', $ids);
    return \Avos\Database\queryAssoc("SELECT * FROM ArticleBlogPosts WHERE id IN ($ids)");
}



function getRelatedVideos($count=8)
{
	\Avos\Database\disableCache();
	$video = getTemplateValue('video');
	#$tags = explode(',', $video['tags']);
	$tags = $video['tags'];
	$score = count($tags);
	$matches = array();

	//
	// rank matching videos based on tags from 1st to last
	//

	foreach ($tags as $t) {
		$videos = \Avos\Database\queryAssoc("SELECT videosId FROM videosTags WHERE name=:tag AND videosId!=:id",
						array(
							'tag' => $t,
							'id' => $video['id']
						)
					);

		foreach ($videos as $v) {
			if (!isset($matches[$v['videosId']])) {
				$matches[$v['videosId']] = $score;
			}
			else {
				$matches[$v['videosId']] += $score;
			}
		}

		$score--;
	}

	//
	// sort by values highest to lowest then truncate
	//

	arsort($matches);
	$ids = array_slice(array_keys($matches), 0, $count);

	//
	// now extract the actual videos data
	//

	$q = 'id=' . implode('|', $ids);
	$videos = getVideos($q);

	return $videos;
}

function getPhotos($q='', $a=array())
{
	global $_AVOS;

    if (!is_array($q)) {
        parse_str($q, $arr);
    }
    else {
        $arr = $q;
    }

	// remove attribs that arent part of the sql filtering

	foreach (array('count', 'sort') as $f) {
		if (isset($arr[$f])) {
			$a[$f] = $arr[$f];
			unset($arr[$f]);
		}
	}

	// corner case where url type=webcams and the userfn calls getPhotos()

	$type = $_AVOS['page']['type'];
	$_AVOS['page']['type'] = 'photos';

	$sql = \Avos\Utils\query2sql($arr);
	$photos = \Avos\Routes\photos($a, $sql);

	$_AVOS['page']['type'] = $type;

	if (isset($photos['args']) && isset($photos['args']['photos'])) {
		return $photos['args']['photos'];
	}
	else {
		return array();
	}
}

function getRelatedPhotos($count=8)
{
	\Avos\Database\disableCache();
	$photo = getTemplateValue('photos');
	#$tags = explode(',', $video['tags']);
	$tags = $photo['tags'];
	$score = count($tags);
	$matches = array();

	//
	// rank matching videos based on tags from 1st to last
	//

	foreach ($tags as $t) {
		$photos = \Avos\Database\queryAssoc("SELECT photosId FROM photosTags WHERE name=:tag AND photosId!=:id",
						array(
							'tag' => $t,
							'id' => $photo['id']
						)
					);

        foreach ($photos as $v) {
			if (!isset($matches[$v['photosId']])) {
				$matches[$v['photosId']] = $score;
            }
			else {
				$matches[$v['photosId']] += $score;
            }
		}

		$score--;
	}


//







    

	//
	// sort by values highest to lowest then truncate
	//

	arsort($matches);
	$ids = array_slice(array_keys($matches), 0, $count);

	//
	// now extract the actual photos data
	//

	$q = 'id=' . implode('|', $ids);
    $photos = getPhotos($q);

	return $photos;
}




function formatTime($seconds) {
    if ($seconds < 60) {
        // Less than a minute
        return $seconds . ' seconds';
    } elseif ($seconds < 3600) {
        // Less than an hour
        $minutes = floor($seconds / 60);
        return $minutes . ' minutes';
    } else {
        // One hour or more
        $hours = floor($seconds / 3600);
        return $hours . ' hours';
    }
}


function thousand_format($number) {
    $number = (int) preg_replace('/[^0-9]/', '', $number);
    if ($number >= 1000) {
        $rn = round($number);
        $format_number = number_format($rn);
        $ar_nbr = explode(',', $format_number);
        $x_parts = array('K', 'M', 'B', 'T', 'Q');
        $x_count_parts = count($ar_nbr) - 1;
        $dn = $ar_nbr[0] . ((int) $ar_nbr[1][0] !== 0 ? '.' . $ar_nbr[1][0] : '');
        $dn .= $x_parts[$x_count_parts - 1];
        return $dn;
    }

    return $number;
}



function getPositionWithSuffix($position) {
    if ($position == 1) {
        return $position . "st";
    } elseif ($position == 2) {
        return $position . "nd";
    } elseif ($position == 3) {
        return $position . "rd";
    } else {
        return $position . "th";
    }
}






function truncate($str, $len)
{
    return \Avos\Utils\truncate($str, $len);
}
